%% Transmission problem for the Helmholtz equation
% We compute the scattered wave by an electromagnetic inclusion
% Here we assume the inclusion is a circular disk so that we can compare
% the numerical solution with the exact analytical one.
% But it is applicable for general shaped inclusions. 

clear all;
close all;
clc;



%% Make a unit disk 
r = 1; % radius of a disk
nbPoints = 100; %number of points on the boundary
theta = 2*pi*(0:nbPoints-1)/nbPoints;
D=ellipse(r,r,0,0,nbPoints); 


%% Set frequency and material parameters
omega=2;
eps_m = 1; eps_c = 1;
mu_m = 1; mu_c = 5;
k_m=omega*sqrt(eps_m*mu_m);
k_c=omega*sqrt(eps_c*mu_c);

%% Solving the integral equation
A11 = SingleL_Helm(k_c,D,D);
A12 = -SingleL_Helm(k_m,D,D);
A21 = (1/mu_c)*( (-1/2)*eye(D.nbPoints) + KDstar_Helm(k_c,D) );
A22 = (-1/mu_m)*( (+1/2)*eye(D.nbPoints) + KDstar_Helm(k_m,D) );
MatA = [A11, A12 ; A21, A22];

%%%%% We assume that the incident field is given by 
%%%%% J_n(k_m*|x|)*exp(i*n*theta) with n=3.
n=3; 
b1 = besselj(n,k_m*r)*exp(1i*n*theta);
b2 = k_m*1/2*(besselj(n-1, k_m*r) - besselj(n+1, k_m*r))*exp(1i*n*theta);
vecB = [b1.';1/mu_m*b2.'];


% get the density functions \phi and \psi
sol = MatA\vecB; 
phi = sol(1:nbPoints);
psi = sol(nbPoints+1:2*nbPoints);


%% Compute the numerical solution (scattered wave) on the circle |x|=2 
r0=2;
Omega=ellipse(r0,r0,0,0,nbPoints);

us_num = SingleL_Helm(k_m,Omega,D)*psi;



%% Compute exact analytical solution (scattered wave) on the circle |x|=2
Jm = besselj(n,k_m*r);
Jm_deri = 1/2*(besselj(n-1, k_m*r) - besselj(n+1, k_m*r));
Jc = besselj(n,k_c*r);
Jc_deri = 1/2*(besselj(n-1, k_c*r) - besselj(n+1, k_c*r));
H = besselh(n,1,k_m*r);
H_deri = 1/2*(besselh(n-1,1, k_m*r) - besselh(n+1,1, k_m*r));
det = -k_m/mu_m*Jc*H_deri + k_c/mu_c*H*Jc_deri;
a=1/det*(-k_c/mu_c*Jm*Jc_deri + k_m/mu_m*Jc*Jm_deri);

us_theo  = a*besselh(n,1,k_m*r0)*exp(1i*n*theta');


%% Plot exact sol. and numerical sol. for comparison 
figure(1)
hold on
plot(theta, real(us_theo), 'r-');
plot(theta, real(us_num), 'o');
legend('Re(u_{e})','Re(u_{n})','Location','southeast')

figure(2)
hold on
plot(theta, imag(us_theo), 'r-');
plot(theta, imag(us_num), 'o');
legend('Im(u_{e})','Im(u_{n})','Location','southeast')



